﻿using System;
using Startup.Base.Utils;
using Startup.Project.Algorithm;
using Startup.Project.Classes;
using Startup.Project.Utils;

namespace Startup.Project.Interfaces
{
	public class ClassContainer : UiContainer
	{
		private readonly CustomClass _source;
	    private readonly string _dataContext;
		private readonly bool _isCollection;

		public ClassContainer(ClassBase source, string dataContext)
			: base(1)
		{
			if (source == null) 
				throw new ArgumentNullException("source");
			if (source.GetType() == typeof(StandartClass))
				throw new ArgumentException("Container cannot contain a standart type.", "source");

			_source = source is CustomClass ? (CustomClass)source : (CustomClass)((CollectionStandartClass)source).TemplateType;
		    _dataContext = dataContext;
	    	_isCollection = source is CollectionStandartClass;
		}

		internal override string SaveToString(UiGridCombinerHelper gridContext)
		{
			if (_isCollection)
			{
				PropertyContainer temp = new PropertyContainer(new ReadonlyPropertyDefinition(new CollectionStandartClass(_source), _dataContext, false));
				return temp.SaveToString(gridContext);
			}
			else
			{
				// properties
				TabStringBuilder innerCode = new TabStringBuilder();
				gridContext.GridIn();

				_source.Properties.ForEach(x =>
				{
					var propertyContainer = new PropertyContainer(x);
					innerCode.AddLine(propertyContainer.SaveToString(gridContext));
				});

				int rowCount = gridContext.CurrentRow;
				gridContext.GridOut();

				// total container
				TabStringBuilder allCode = new TabStringBuilder();

				allCode.AddLine("<Grid Grid.Row=\"{0}\" Grid.Column=\"{1}\" >", gridContext.CurrentRow, gridContext.CurrentColumn);
//				allCode.AddLine("<Grid Grid.Row=\"{0}\" Grid.Column=\"{1}\" {2}>", gridContext.CurrentRow, gridContext.CurrentColumn,
//					string.IsNullOrEmpty(_dataContext) ? string.Empty : "DataContext=\"{Binding " + _dataContext + "}\"");
				allCode.Tab().AddLine("<Grid.RowDefinitions>");

				for (int index = 0; index < rowCount; index++)
					allCode.Tab().Tab().AddLine("<RowDefinition Height=\"{0}\" />", index == rowCount - 1 ? "*" : "Auto");

				allCode.Tab().AddLine("</Grid.RowDefinitions>");
				allCode.Tab().AddLine("<Grid.ColumnDefinitions>");
				allCode.Tab().Tab().AddLine("<ColumnDefinition Width=\"Auto\" />");
				allCode.Tab().Tab().AddLine("<ColumnDefinition Width=\"*\" />");
				allCode.Tab().AddLine("</Grid.ColumnDefinitions>");

				allCode.AddLine(innerCode.ToString());
				allCode.AddLine("</Grid>");

				return allCode.ToString();
			}
		}
	}
}
