﻿using System.Windows.Controls;
using Startup.Base.Utils;
using Startup.Project.Utils;

namespace Startup.Project.Interfaces
{
	public class GridContainer : UiContainer
	{
		private readonly Orientation _orientation;
		private readonly int[] _parts;

		public GridContainer(Orientation orientation, params int[] parts)
			: base(parts.Length)
		{
			_orientation = orientation;
			_parts = parts;
		}

		internal override string SaveToString(UiGridCombinerHelper gridContext)
		{
			TabStringBuilder builder = new TabStringBuilder();

			builder.AddLine("<Grid Grid.Row=\"{0}\" Grid.Column=\"{1}\">", gridContext.CurrentRow, gridContext.CurrentColumn);
			gridContext.GridIn();

			if (_orientation == Orientation.Vertical)
			{
				builder.Tab().AddLine("<Grid.RowDefinitions>");

				for (int index = 0; index < _parts.Length; index++)
				{
					int partSize = _parts[index];

					if (partSize > 0)
					{
						builder.Tab().Tab().AddLine("<RowDefinition Height=\"{0}*\" />", partSize.ToString());
					}
					else
					{
						builder.Tab().Tab().AddLine("<RowDefinition Height=\"Auto\" />");
					}
				}

				builder.Tab().AddLine("</Grid.RowDefinitions>");
			}
			else if (_orientation == Orientation.Horizontal)
			{
				builder.Tab().AddLine("<Grid.ColumnDefinitions>");

				for (int index = 0; index < _parts.Length; index++)
				{
					int partSize = _parts[index];

					if (partSize > 0)
					{
						builder.Tab().Tab().AddLine("<ColumnDefinition Width=\"{0}*\" />", partSize.ToString());
					}
					else
					{
						builder.Tab().Tab().AddLine("<ColumnDefinition Width=\"Auto\" />");
					}
				}
				
				builder.Tab().AddLine("</Grid.ColumnDefinitions>");
			}

			// content
			for (int index = 0; index < Children.Length; index++)
			{
				UiContainer child = Children[index];

				if (child == null)
					continue;
				
				string childContent = child.SaveToString(gridContext);
				builder.AddLine(childContent);

				if (_orientation == Orientation.Horizontal)
					gridContext.NextColumn();
				else gridContext.NextRow();
			}
			//

			builder.AddLine("</Grid>");
			gridContext.GridOut();

			gridContext.NextRow();

			return builder.ToString();
		}
	}
}
