﻿using System;
using Startup.Base.Utils;
using Startup.Project.Algorithm;
using Startup.Project.Classes;
using Startup.Project.Utils;

namespace Startup.Project.Interfaces
{
	internal class PropertyContainer : UiContainer
	{
		private readonly ReadonlyPropertyDefinition _source;

		public PropertyContainer(ReadonlyPropertyDefinition source) 
			: base(1)
		{
			if (source == null) 
				throw new ArgumentNullException("source");
			
			_source = source;
		}

		internal override string SaveToString(UiGridCombinerHelper gridContext)
		{
			TabStringBuilder code = new TabStringBuilder();

			if (_source.ReturnedType is CollectionStandartClass)
			{
//				code.AddLine("<TextBlock Grid.Row=\"{0}\" Grid.Column=\"{1}\" Text=\"{2}\"/>", gridContext.CurrentRow, gridContext.CurrentColumn, _source.Name);
//				gridContext.NextColumn();
				code.AddLine("<DataGrid ItemsSource=\"{0}\" Grid.Row=\"{1}\" Grid.Column=\"{2}\" Grid.ColumnSpan=\"2\" Background=\"White\" CanUserAddRows=\"False\" CanUserDeleteRows=\"False\" AlternatingRowBackground=\"LightGray\" AutoGenerateColumns=\"True\" GridLinesVisibility=\"None\" />",
					"{Binding " + _source.Name + "}", gridContext.CurrentRow, gridContext.CurrentColumn);
//				gridContext.PreviosColumn();
			}
			else if (_source.ReturnedType is StandartClass)
			{
				if (_source.ReturnedType.Name == typeof(string).Name)
				{
					code.AddLine("<TextBlock Grid.Row=\"{0}\" Grid.Column=\"{1}\" Text=\"{2}\"/>", gridContext.CurrentRow, gridContext.CurrentColumn, _source.Name);
					gridContext.NextColumn();
					code.AddLine("<TextBox Grid.Row=\"{0}\" Grid.Column=\"{1}\" Text=\"{2}\" TextWrapping=\"Wrap\" />", gridContext.CurrentRow, gridContext.CurrentColumn, "{Binding " + _source.Name + ", Mode=TwoWay, UpdateSourceTrigger=PropertyChanged}");
					gridContext.PreviosColumn();
				}
				else if (_source.ReturnedType.Name == typeof(Guid).Name)
				{
					code.AddLine("<TextBlock Grid.Row=\"{0}\" Grid.Column=\"{1}\" Text=\"{2}\"/>", gridContext.CurrentRow, gridContext.CurrentColumn, _source.Name);
					gridContext.NextColumn();
					code.AddLine("<TextBox Grid.Row=\"{0}\" Grid.Column=\"{1}\" Text=\"{2}\"/>", gridContext.CurrentRow, gridContext.CurrentColumn, "{Binding " + _source.Name + ", Mode=TwoWay}");
					gridContext.PreviosColumn();
				}
				else if (_source.ReturnedType.Name == typeof(int).Name)
				{
					code.AddLine("<TextBlock Grid.Row=\"{0}\" Grid.Column=\"{1}\" Text=\"{2}\"/>", gridContext.CurrentRow, gridContext.CurrentColumn, _source.Name);
					gridContext.NextColumn();
					code.AddLine("<xctk:IntegerUpDown Grid.Row=\"{0}\" Grid.Column=\"{1}\" Value=\"{2}\"/>", gridContext.CurrentRow, gridContext.CurrentColumn, "{Binding " + _source.Name + ", Mode=TwoWay}");
					gridContext.PreviosColumn();
				}
				else if (_source.ReturnedType.Name == typeof(decimal).Name)
				{
					code.AddLine("<TextBlock Grid.Row=\"{0}\" Grid.Column=\"{1}\" Text=\"{2}\"/>", gridContext.CurrentRow, gridContext.CurrentColumn, _source.Name);
					gridContext.NextColumn();
					code.AddLine("<xctk:DecimalUpDown Grid.Row=\"{0}\" Grid.Column=\"{1}\" Value=\"{2}\"/>", gridContext.CurrentRow, gridContext.CurrentColumn, "{Binding " + _source.Name + ", Mode=TwoWay}");
					gridContext.PreviosColumn();
				}
				else if (_source.ReturnedType.Name == typeof(double).Name)
				{
					code.AddLine("<TextBlock Grid.Row=\"{0}\" Grid.Column=\"{1}\" Text=\"{2}\"/>", gridContext.CurrentRow, gridContext.CurrentColumn, _source.Name);
					gridContext.NextColumn();
					code.AddLine("<xctk:DoubleUpDown Grid.Row=\"{0}\" Grid.Column=\"{1}\" Value=\"{2}\"/>", gridContext.CurrentRow, gridContext.CurrentColumn, "{Binding " + _source.Name + ", Mode=TwoWay}");
					gridContext.PreviosColumn();
				}
				else if (_source.ReturnedType.Name == typeof(bool).Name)
				{
					code.AddLine("<CheckBox Grid.Row=\"{0}\" Grid.Column=\"{1}\" IsChecked=\"{2}\" Content=\"{3}\" />", gridContext.CurrentRow, gridContext.CurrentColumn, "{Binding " + _source.Name + ", Mode=TwoWay}", _source.Name);
				}
				else
				{
					code.AddLine("<TextBlock Grid.Row=\"{0}\" Grid.Column=\"{1}\" Text=\"{2}\"/>", gridContext.CurrentRow, gridContext.CurrentColumn, _source.Name);
				}
			}
			else if (_source.ReturnedType is CustomClass)
			{
				code.AddLine("<TextBlock Grid.Row=\"{0}\" Grid.Column=\"{1}\" Text=\"{2}\"/>", gridContext.CurrentRow, gridContext.CurrentColumn, _source.Name);
				gridContext.NextColumn();
				
				CustomClass target = (CustomClass) _source.ReturnedType;
				ClassContainer container = new ClassContainer(target, _source.Name);
				code.AddLine(container.SaveToString(gridContext));

				gridContext.PreviosColumn();
			}
			else
			{
				throw new Exception(string.Format("'{0}' type is unknown for this algorithm.", _source.ReturnedType.GetType()));
			}

			gridContext.NextRow();

			return code.ToString();
		}
	}
}
