﻿using System;
using System.Windows.Controls;
using Startup.Project.Utils;

namespace Startup.Project.Interfaces
{
	public abstract class UiContainer
	{
		private readonly UiContainer[] _children;
        
		protected UiContainer(int childrenCount)
		{
			if (childrenCount < 0)
				throw new ArgumentException("Count of child cannot be negative.");

			_children = new UiContainer[childrenCount];
		}

		public void AddChild(UiContainer child, int index)
		{
			if (child == null) 
				throw new ArgumentNullException("child");
			if (index < 0)
				throw new ArgumentException("Index of child cannot be less than zero.");
			if (index >= _children.Length)
				throw new ArgumentException("Index of child cannot be more than array size.");
			
			_children[index] = child;
		}

		public GridContainer AddGrid(int index, Orientation orientation, params int[] parts)
		{
			GridContainer grid = new GridContainer(orientation, parts);
			AddChild(grid, index);

			return grid;
		}

		internal abstract string SaveToString(UiGridCombinerHelper gridContext);

		public UiContainer[] Children
		{
			get { return _children; }
		}
	}
}
