﻿using System;
using System.Collections.Generic;
using System.IO;
using Startup.Project.Algorithm;
using Startup.Project.Classes;

namespace Startup.Project.Interfaces
{
	public abstract class ViewBase : UiContainer
	{
		internal const string Extension = ".xaml";

		private readonly string _name;
		private readonly string _parentNamespace;
		private readonly CustomClass _viewModelClass;
		private readonly CustomClass _modelClass;
		private readonly string _relativeParentPath;

		protected ViewBase(string name, string parentNamespace, CustomClass viewModelClass, CustomClass modelClass, string relativeParentPath)
			: base(1)
		{
			if (viewModelClass == null) 
				throw new ArgumentNullException("viewModelClass");
			if (modelClass == null) 
				throw new ArgumentNullException("modelClass");
			if (string.IsNullOrEmpty(name)) 
				throw new ArgumentNullException("name");
			if (string.IsNullOrEmpty(parentNamespace))
				throw new ArgumentNullException("parentNamespace");
			
			_name = name;
			_parentNamespace = parentNamespace;
			_viewModelClass = viewModelClass;
			_modelClass = modelClass;
			_relativeParentPath = relativeParentPath;
		}

		internal string Name
		{
			get { return _name; }
		}

		internal CustomClass ViewModelClass
		{
			get { return _viewModelClass; }
		}

		public string RelativePath
		{
			get { return Path.Combine(_relativeParentPath, Name + Extension); }
		}

		internal CustomClass ModelClass
		{
			get { return _modelClass; }
		}

		public string ParentNamespace
		{
			get { return _parentNamespace; }
		}

		internal abstract void Save(string directoryPath);
		public abstract void AddCommands(List<CommandDefinition> commands);
	}
}
