﻿using System;
using System.IO;
using Startup.Base.Exceptions;
using Startup.Base.Utils;
using Startup.Project.Interfaces;

namespace Startup.Project.Project
{
	public class App
	{
		private readonly string _parentNamespace;
		public WindowContainer StartWindow { get; set; }

		public App(string parentNamespace)
		{
			if (string.IsNullOrEmpty(parentNamespace))
				throw new ArgumentNullException("parentNamespace");
			
			_parentNamespace = parentNamespace;
		}

		public void Save(string directoryPath)
		{
			if (string.IsNullOrEmpty(directoryPath))
				throw new ArgumentNullException("directoryPath");
			if (StartWindow == null)
				throw new IncorrectStateException("This application cannot be saved without start window.");

			TabStringBuilder appCodeContent = new TabStringBuilder();
			appCodeContent.AddLine("using {0}.ShellModule;", _parentNamespace);
			appCodeContent.AddLine();
			appCodeContent.AddLine("namespace {0}", _parentNamespace);
			appCodeContent.AddLine("{");
			appCodeContent.Tab().AddLine("public partial class App");
			appCodeContent.Tab().AddLine("{");
			appCodeContent.Tab().Tab().AddLine("private readonly DomainModel _domainModel = new DomainModel();");
			appCodeContent.AddLine();
			appCodeContent.Tab().Tab().AddLine("public static ShellViewModel DomainModel");
			appCodeContent.Tab().Tab().AddLine("{");
			appCodeContent.Tab().Tab().Tab().AddLine("get");
			appCodeContent.Tab().Tab().Tab().AddLine("{");
			appCodeContent.Tab().Tab().Tab().Tab().AddLine("if (Current == null)");
			appCodeContent.Tab().Tab().Tab().Tab().Tab().AddLine("return null;");
			appCodeContent.AddLine();
			appCodeContent.Tab().Tab().Tab().Tab().AddLine("if (Current is App)");
			appCodeContent.Tab().Tab().Tab().Tab().Tab().AddLine("return new ShellViewModel(new ShellModel(((App)Current)._domainModel));");
			appCodeContent.AddLine();
			appCodeContent.Tab().Tab().Tab().Tab().AddLine("return null;");
			appCodeContent.Tab().Tab().Tab().AddLine("}");
			appCodeContent.Tab().Tab().AddLine("}");
			appCodeContent.Tab().AddLine("}");
			appCodeContent.AddLine("}");

			string appCodePath = Path.Combine(directoryPath, "App.xaml.cs");
			File.WriteAllText(appCodePath, appCodeContent.ToString());

			TabStringBuilder appXamlContent = new TabStringBuilder();
			appXamlContent.AddLine("<Application x:Class=\"{0}.App\"", _parentNamespace);
			appXamlContent.Tab().AddLine("xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"");
			appXamlContent.Tab().AddLine("xmlns:x=\"http://schemas.microsoft.com/winfx/2006/xaml\"");
			appXamlContent.Tab().AddLine("StartupUri=\"{0}\">", StartWindow.RelativePath);
			appXamlContent.Tab().AddLine("<Application.Resources>");
			appXamlContent.Tab().AddLine("</Application.Resources>");
			appXamlContent.AddLine("</Application>");

			string appXamlPath = Path.Combine(directoryPath, "App.xaml");
			File.WriteAllText(appXamlPath, appXamlContent.ToString());
		}
	}
}
