﻿using System;
using System.IO;
using System.Linq;
using System.Xml.Linq;
using Startup.Project.Classes;
using Startup.Project.Interfaces;
using Xceed.Wpf.Toolkit;

namespace Startup.Project.Project
{
	public class Project : Folder
	{
		private readonly App _app;

		public Project(string name)
			: base(name, string.Empty, string.Empty)
		{
			_app = new App(name);
		}

	    public App App
	    {
	        get { return _app; }
	    }

	    internal override string RelativePath
		{
			get { return string.Empty; }
		}

		public override void Save(string directory)
		{
			// save classes and views
			base.Save(directory);
			
			// save App instance
			_app.Save(directory);

			// generate project
			XNamespace xmlns = @"http://schemas.microsoft.com/developer/msbuild/2003";
			XElement classesElement = new XElement(xmlns + "ItemGroup"); 
			XElement viewsElement = new XElement(xmlns + "ItemGroup"); 

			XDocument projectContent = new XDocument(
				new XElement(xmlns + "Project",
					new XAttribute("ToolsVersion", "4.0"),
					new XAttribute("DefaultTargets", "Build"),
					new XElement(xmlns + "PropertyGroup",
						new XElement(xmlns + "Configuration",
							new XAttribute("Condition", " '$(Configuration)' == '' "), "Debug"),
						new XElement(xmlns + "Platform",
							new XAttribute("Condition", " '$(Platform)' == '' "), "AnyCPU"),
						new XElement(xmlns + "ProductVersion", "8.0.30703"),
						new XElement(xmlns + "SchemaVersion", "2.0"),
						new XElement(xmlns + "ProjectGuid", Guid.NewGuid().ToString()),
						new XElement(xmlns + "OutputType", "WinExe"),
						new XElement(xmlns + "AppDesignerFolder", "Properties"),
						new XElement(xmlns + "RootNamespace", Namespace),
						new XElement(xmlns + "AssemblyName", Name),
						new XElement(xmlns + "TargetFrameworkVersion", "v4.0"),
						new XElement(xmlns + "FileAlignment", "512"),
						new XElement(xmlns + "ProjectTypeGuids", "{60dc8134-eba5-43b8-bcc9-bb4bc16c2548};{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}"),
						new XElement(xmlns + "WarningLevel", "4")),
					new XElement(xmlns + "PropertyGroup",
						new XAttribute("Condition", " '$(Configuration)|$(Platform)' == 'Debug|AnyCPU' "),
						new XElement(xmlns + "DebugSymbols", "true"),
						new XElement(xmlns + "DebugType", "full"),
						new XElement(xmlns + "Optimize", "false"),
						new XElement(xmlns + "OutputPath", @"bin\Debug\"),
						new XElement(xmlns + "DefineConstants", "DEBUG;TRACE"),
						new XElement(xmlns + "ErrorReport", "prompt"),
						new XElement(xmlns + "WarningLevel", "4")),
					new XElement(xmlns + "PropertyGroup",
						new XAttribute("Condition", " '$(Configuration)|$(Platform)' == 'Release|AnyCPU' "),
						new XElement(xmlns + "DebugType", "pdbonly"),
						new XElement(xmlns + "OutputPath", @"bin\Release\"),
						new XElement(xmlns + "DefineConstants", "TRACE"),
						new XElement(xmlns + "ErrorReport", "prompt"),
						new XElement(xmlns + "WarningLevel", "4")),
					new XElement(xmlns + "ItemGroup",
						new XElement(xmlns + "Reference", new XAttribute("Include", "System")),
						new XElement(xmlns + "Reference", new XAttribute("Include", "System.Core")),
						new XElement(xmlns + "Reference", new XAttribute("Include", "System.Xml.Linq")),
						new XElement(xmlns + "Reference", 
							new XAttribute("Include", "System.Data.DataSetExtensions"),
							new XElement(xmlns + "RequiredTargetFramework", "4.0")),
						new XElement(xmlns + "Reference", new XAttribute("Include", "Microsoft.CSharp")),
						new XElement(xmlns + "Reference", new XAttribute("Include", "System.Data")),
						new XElement(xmlns + "Reference", new XAttribute("Include", "System.Xml")),
						new XElement(xmlns + "Reference", new XAttribute("Include", "System.Xaml")),
						new XElement(xmlns + "Reference", new XAttribute("Include", "WindowsBase")),
						new XElement(xmlns + "Reference", new XAttribute("Include", "PresentationCore")),
						new XElement(xmlns + "Reference", new XAttribute("Include", "PresentationFramework")),
						new XElement(xmlns + "Reference", new XAttribute("Include", "Startup.Base"),
							new XElement(xmlns + "HintPath", @"..\Startup.Base.dll")),
						new XElement(xmlns + "Reference", new XAttribute("Include", "Startup.MVVM"),
							new XElement(xmlns + "HintPath", @"..\Startup.MVVM.dll")),
						new XElement(xmlns + "Reference", new XAttribute("Include", "WPFToolkit.Extended"),
							new XElement(xmlns + "HintPath", typeof(IntegerUpDown).Assembly.Location))),
					new XElement(xmlns + "ItemGroup",
						new XElement(xmlns + "Compile", 
							new XAttribute("Include", "App.xaml.cs"),
							new XElement(xmlns + "DependentUpon", "App.xaml"),
							new XElement(xmlns + "SubType", "Code")),
						new XElement(xmlns + "ApplicationDefinition",
							new XAttribute("Include", "App.xaml"),
							new XElement(xmlns + "Generator", "MSBuild:Compile"),
							new XElement(xmlns + "SubType", "Designer"))),
					classesElement,
					viewsElement,
					new XElement(xmlns + "Import", new XAttribute("Project", @"$(MSBuildToolsPath)\Microsoft.CSharp.targets"))));

			projectContent.Declaration = new XDeclaration("1.0", "utf-8", "false");

			// referenses of classes
			var classDescription = from classItem in GetAllCustomClasses()
			                       select classItem.Class.IsPartOfXaml ?
										
								   new XElement(xmlns + "Compile", 
										new XAttribute("Include", classItem.RelativePath),
										new XElement(xmlns + "DependentUpon", Path.GetFileNameWithoutExtension(Path.GetFileNameWithoutExtension(classItem.RelativePath)) + ViewBase.Extension),
										new XElement(xmlns + "SubType", "Code")) :

									new XElement(xmlns + "Compile", new XAttribute("Include", classItem.RelativePath));

			foreach (var xElement in classDescription)
				classesElement.Add(xElement);

			// references of views
			var viewDescription = from viewItem in GetAllViews()
								  select new XElement(xmlns + "Page",
										new XAttribute("Include", viewItem.RelativePath),
										new XElement(xmlns + "Generator", "MSBuild:Compile"),
										new XElement(xmlns + "SubType", "Designer"));

			foreach (var xElement in viewDescription)
				viewsElement.Add(xElement);

			string projectPath = Path.Combine(directory, Name + ".csproj");
			File.WriteAllText(projectPath, projectContent.ToString());
		}
		
		public override string ToString()
		{
			return Name;
		}

		public override ClassBase GetClass(string typeName)
		{
			ClassBase child = base.GetClass(typeName);

			if (child != null)
				return child;

			Type standartType = Type.GetType(typeName);

			if (standartType != null)
				return new StandartClass(standartType.Name, standartType.Namespace);

			throw new Exception(string.Format("Cannot find the class '{0}'.", typeName));
		}

		public void SetStartWindow(WindowContainer window)
		{
			_app.StartWindow = window;
		}


	}
}
