﻿using System.Collections.Generic;
using System.Linq;
using Startup.Base.Exceptions;

namespace Startup.Project.Utils
{
	internal class UiGridCombinerHelper
	{
		private readonly List<UiGridCombinerHelperState> _stateStack = new List<UiGridCombinerHelperState>();
		private UiGridCombinerHelperState _currentState = new UiGridCombinerHelperState();

		internal void GridIn()
		{
			_stateStack.Add(_currentState);
			_currentState = new UiGridCombinerHelperState();
		}

		internal void GridOut()
		{
			if (_stateStack.Count == 0)
				throw new IncorrectStateException("There is no any entered grid.");

			_currentState = _stateStack.Last();
			_stateStack.Remove(_currentState);
		}

		public int CurrentRow
		{
			get { return _currentState.CurrentRow; }
		}

		public int CurrentColumn
		{
			get { return _currentState.CurrentColumn; }
		}

		internal void SetRow(int row)
		{
			_currentState.SetRow(row);
		}

		internal void SetColumn(int column)
		{
			_currentState.SetColumn(column);
		}

		internal void NextRow()
		{
			_currentState.NextRow();
		}

		internal void NextColumn()
		{
			_currentState.NextColumn();
		}

		internal void PreviosRow()
		{
			_currentState.PreviosRow();
		}

		internal void PreviosColumn()
		{
			_currentState.PreviosColumn();
		}

		private class UiGridCombinerHelperState
		{
			private int _currentRow = 0;
			private int _currentColumn = 0;

			public int CurrentRow
			{
				get { return _currentRow; }
			}

			public int CurrentColumn
			{
				get { return _currentColumn; }
			}

			internal void SetRow(int row)
			{
				_currentRow = row;
			}

			internal void SetColumn(int column)
			{
				_currentColumn = column;
			}

			internal void NextRow()
			{
				_currentRow++;
			}

			internal void NextColumn()
			{
				_currentColumn++;
			}

			internal void PreviosRow()
			{
				if (_currentRow == 0)
					throw new IncorrectStateException("Row cannot be less than aero.");

				_currentRow--;
			}

			internal void PreviosColumn()
			{
				if (_currentColumn == 0)
					throw new IncorrectStateException("Column cannot be less than aero.");

				_currentColumn--;
			}
		}
	}
}
